#include "GameLib/Framework.h"

#include "Game/State.h"
#include "Game/StaticObject.h"
#include "Game/DynamicObject.h"
#include "SoundManager.h"
#include "Image.h"

using namespace GameLib;

namespace {

//}bv̍L
static const int WIDTH = 19;
static const int HEIGHT = 15;
//ep[^
static const int EXPLOSION_TIME = 180; //3b
static const int EXPLOSION_LIFE = 60; //1b

//KXe[Wf[^
struct StageData{
	int mEnemyNumber; //G̐
	int mBrickRate; //(p[Zg)
	int mItemPowerNumber; //ACe̐
	int mItemBombNumber; //eACe̐
};

static StageData gStageData[] = {
	{ 2, 50, 10, 10, },
	{ 3, 70, 1, 0, },
	{ 5, 30, 0, 1, },
};

} //namespace{}

State::State( int stageID ) : 
mImage( 0 ),
mDynamicObjects( 0 ),
mDynamicObjectNumber( 0 ),
mStageID( stageID ){
	Framework f = Framework::instance(); //ŉxĝ
	mStaticObjects.setSize( WIDTH, HEIGHT );

	mImage = new Image( "data/image/bakudanBitoImage.dds" );

	const StageData& stageData = gStageData[ mStageID ];
	int n = HEIGHT * WIDTH; //}Xڂ̐

	//̃ubNL^B
	unsigned* brickList = new unsigned[ n ];
	int brickNumber = 0; //{ɃKɂȂJEg
	unsigned* floorList = new unsigned[ n ];
	int floorNumber = 0;

	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			StaticObject& o = mStaticObjects( x, y );
			if ( x == 0 || y == 0 || ( x == WIDTH-1 ) || ( y == HEIGHT-1 ) ){
				o.setFlag( StaticObject::FLAG_WALL );
			}else if ( ( x % 2 == 0 ) && ( y % 2 == 0 ) ){ //RN[g
				o.setFlag( StaticObject::FLAG_WALL );
			}else if ( y + x < 4 ){
				//3}X͏
			}else if ( ( stageID == 0 ) && ( y + x > ( WIDTH + HEIGHT - 6 ) ) ){
				; //lpȂE3}X󂯂B
			}else{ //c͗B100ʃTCRUČ߂
				if ( f.getRandom( 100 ) < stageData.mBrickRate  ){
					o.setFlag( StaticObject::FLAG_BRICK );
					//L^ĂB
					brickList[ brickNumber ] = ( x << 16 ) + y;
					++brickNumber;
				}else{
					floorList[ floorNumber ] = ( x << 16 ) + y;
					++floorNumber;
				}
			}
		}
	}
	//ɃACed
	int powerNumber = stageData.mItemPowerNumber;
	int bombNumber = stageData.mItemBombNumber;
	//́AXgiԖڂKȂ̂ƎւāAɃACeB
	for ( int i = 0; i < powerNumber + bombNumber; ++i ){
 		int swapped = f.getRandom( brickNumber - 1 - i ) + i; //AƎւBłȂƂłɓꂽ}XoĂĂ܂B
		unsigned t = brickList[ i ];
		brickList[ i ] = brickList[ swapped ];
		brickList[ swapped ] = t;

		int x = brickList[ i ] >> 16;
		int y = brickList[ i ] & 0xffff;
		StaticObject& o = mStaticObjects( x, y );
		if ( i < powerNumber ){
			o.setFlag( StaticObject::FLAG_ITEM_POWER );
		}else{
			o.setFlag( StaticObject::FLAG_ITEM_BOMB );
		}
	}
	SAFE_DELETE_ARRAY( brickList );

	//IIuWFNgm
	int playerNumber = ( mStageID == 0 ) ? 2 : 1;
	int enemyNumber = stageData.mEnemyNumber;
	mDynamicObjectNumber = playerNumber + enemyNumber;
	mDynamicObjects = new DynamicObject[ mDynamicObjectNumber ];

	//vC[zu
	mDynamicObjects[ 0 ].set( 1, 1, DynamicObject::TYPE_PLAYER );
	mDynamicObjects[ 0 ].mPlayerID = 0;
	if ( mStageID == 0 ){
		mDynamicObjects[ 1 ].set( WIDTH-2, HEIGHT-2, DynamicObject::TYPE_PLAYER );
		mDynamicObjects[ 1 ].mPlayerID = 1;
	}
	//ɓGdށB̓ACeƂقƂǓ
	for ( int i = 0; i < enemyNumber; ++i ){
 		int swapped = f.getRandom( floorNumber - 1 - i ) + i;
		unsigned t = floorList[ i ];
		floorList[ i ] = floorList[ swapped ];
		floorList[ swapped ] = t;

		int x = floorList[ i ] >> 16;
		int y = floorList[ i ] & 0xffff;
		mDynamicObjects[ playerNumber + i ].set( x, y, DynamicObject::TYPE_ENEMY );
	}
	SAFE_DELETE_ARRAY( floorList );
}

State::~State(){
	SAFE_DELETE( mImage );
	SAFE_DELETE_ARRAY( mDynamicObjects );
}

void State::draw() const {
	//wi`
	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			mStaticObjects( x, y ).draw( x, y, mImage );
		}
	}
	//Oi`
	for ( int i = 0; i < mDynamicObjectNumber; ++i ){
		mDynamicObjects[ i ].draw( mImage );
	}
	//`
	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			mStaticObjects( x, y ).drawExplosion( x, y, mImage );
		}
	}
}

void State::update(){
	//ȅ
	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			StaticObject& o = mStaticObjects( x, y );
			if ( o.checkFlag( StaticObject::FLAG_BOMB ) ){
				//1.ẽJEgXV
				++o.mCount;
				//2.jJnAI
				if ( o.checkFlag( StaticObject::FLAG_EXPLODING ) ){ //Δ
					if ( o.mCount == EXPLOSION_LIFE ){ //IɂȂ
						o.resetFlag( StaticObject::FLAG_EXPLODING | StaticObject::FLAG_BOMB );
						o.mCount = 0;
					}
				}else{ //j
					if ( o.mCount == EXPLOSION_TIME ){ //jɂȂ
						o.setFlag( StaticObject::FLAG_EXPLODING );
						o.mCount = 0;
						SoundManager::instance()->playSe( SoundManager::SE_EXPLOSION );
					}else if ( o.checkFlag( StaticObject::FLAG_FIRE_X | StaticObject::FLAG_FIRE_Y ) ){ //U
						o.setFlag( StaticObject::FLAG_EXPLODING );
						o.mCount = 0;
						SoundManager::instance()->playSe( SoundManager::SE_EXPLOSION );
					}
				}
			}else if ( o.checkFlag( StaticObject::FLAG_BRICK ) ){ //K̏ꍇĂ肪Kv
				if ( o.checkFlag( StaticObject::FLAG_FIRE_X | StaticObject::FLAG_FIRE_Y ) ){ //΂Ă
					++o.mCount; //Õt[ł΂Ȃ̂ŔOɃCNg
					if ( o.mCount == EXPLOSION_LIFE ){
						o.mCount = 0;
						o.resetFlag( StaticObject::FLAG_BRICK ); //Ă
					}
				}
			}
			//3.͖t[uȂ̂ŁAB
			o.resetFlag( StaticObject::FLAG_FIRE_X | StaticObject::FLAG_FIRE_Y );
		}
	}
	//ݒu
	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			if ( mStaticObjects( x, y ).checkFlag( StaticObject::FLAG_EXPLODING ) ){
				setFire( x, y );
			}
		}
	}

	//1P,2P̐ݒueJEg
	int bombNumber[ 2 ];
	bombNumber[ 0 ] = bombNumber[ 1 ] = 0;
	for ( int y = 0; y < HEIGHT; ++y ){
		for ( int x = 0; x < WIDTH; ++x ){
			const StaticObject& o = mStaticObjects( x, y );
			if ( o.checkFlag( StaticObject::FLAG_BOMB ) ){
				++bombNumber[ o.mBombOwner->mPlayerID ];
			}
		}
	}
	//_Ci~bNIuWFNgŃ[v
	for ( int i = 0; i < mDynamicObjectNumber; ++i ){
		DynamicObject& o = mDynamicObjects[ i ];
		if ( o.isDead() ){ //łBIB
			continue;
		}
		//ueƐڐGĂ邩`FbN
		for ( int j = 0; j < 2; ++j ){
			if ( o.mLastBombX[ j ] >= 0 ){ //0ȏȂ牽ĂB
				if ( !o.isIntersectWall( o.mLastBombX[ j ], o.mLastBombY[ j ] ) ){
					o.mLastBombX[ j ] = o.mLastBombY[ j ] = -1;
				}
			}
		}
		//݃Z擾
		int x, y;
		o.getCell( &x, &y );
		//𒆐SƂZ̒ǂTĔzɊi[
		int wallsX[ 9 ];
		int wallsY[ 9 ];
		int wallNumber = 0;
		for ( int i = 0; i < 3; ++i ){
			for ( int j = 0; j < 3; ++j ){
				int tx = x + i - 1;
				int ty = y + j - 1;
				const StaticObject& so = mStaticObjects( tx, ty );
				if ( so.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK | StaticObject::FLAG_BOMB ) ){ //ǂ
					bool myBomb0 = ( o.mLastBombX[ 0 ] == tx ) && ( o.mLastBombY[ 0 ] == ty );
					bool myBomb1 = ( o.mLastBombX[ 1 ] == tx ) && ( o.mLastBombY[ 1 ] == ty );
					if ( !myBomb0 && !myBomb1 ){ //ueȂ
						wallsX[ wallNumber ] = x + i - 1;
						wallsY[ wallNumber ] = y + j - 1;
						++wallNumber;
					}
				}
			}
		}
		//ǃXgnĈړ
		o.move( wallsX, wallsY, wallNumber );
		//ړ̈ʒuŎ9}XƏՓ˔肵Ă낢Ȕ
		for ( int i = 0; i < 3; ++i ){
			for ( int j = 0; j < 3; ++j ){
				StaticObject& so = mStaticObjects( x + i - 1, y + j - 1 );
				if ( o.isIntersectWall( x + i - 1, y + j - 1 ) ){ //GĂ܂
					if ( so.checkFlag( StaticObject::FLAG_FIRE_X | StaticObject::FLAG_FIRE_Y ) ){
						o.die(); //Ăꂽ
					}else if ( !so.checkFlag( StaticObject::FLAG_BRICK ) ){ //ɂȂĂACe
						if ( so.checkFlag( StaticObject::FLAG_ITEM_POWER ) ){
							so.resetFlag( StaticObject::FLAG_ITEM_POWER );
							++o.mBombPower;
						}else if ( so.checkFlag( StaticObject::FLAG_ITEM_BOMB ) ){
							so.resetFlag( StaticObject::FLAG_ITEM_BOMB );
							++o.mBombNumber;
						}
					}
				}
			}
		}
		//ړZԍ擾
		o.getCell( &x, &y );
		//eu
		if ( o.hasBombButtonPressed() ){ //eݒu{^Ă
			if ( bombNumber[ o.mPlayerID ] < o.mBombNumber ){ //eől
				StaticObject& so = mStaticObjects( x, y );
				if ( !so.checkFlag( StaticObject::FLAG_BOMB ) ){ //eȂ
					so.setFlag( StaticObject::FLAG_BOMB );
					so.mBombOwner = &o;
					so.mCount = 0;

					//ueʒuXV
					if ( o.mLastBombX[ 0 ] < 0 ){
						o.mLastBombX[ 0 ] = x;
						o.mLastBombY[ 0 ] = y;
					}else{
						o.mLastBombX[ 1 ] = x;
						o.mLastBombY[ 1 ] = y;
					}
					SoundManager::instance()->playSe( SoundManager::SE_SET_BOMB );
				}
			}
		}
	}
	//BGƃvC[̐ڐGB
	for ( int i = 0; i < mDynamicObjectNumber; ++i ){
		for ( int j = i + 1; j < mDynamicObjectNumber; ++j ){
			mDynamicObjects[ i ].doCollisionReactionToDynamic( &mDynamicObjects[ j ] );
		}
	}
}

void State::setFire( int x, int y ){
	StaticObject& o = mStaticObjects( x, y );
	int power = o.mBombOwner->mBombPower;
	int end;
	
	//[v̏͂قƂ4[vňꏏB
	//낤Ǝv΋ʉ͂łBAɂĉ̂킩ɂ֐łĂ܂_B
	//ł4񓯂̂B

	//
	end = ( x - power < 0 ) ? 0 : ( x - power );
	for ( int i = x - 1; i >= end; --i ){
		StaticObject& to = mStaticObjects( i, y );
		to.setFlag( StaticObject::FLAG_FIRE_X );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK | StaticObject::FLAG_BOMB ) ){ //uĂΉ΂͎~܂
			break;
		}else{
			//ACeΖE
			to.resetFlag( StaticObject::FLAG_ITEM_BOMB | StaticObject::FLAG_ITEM_POWER );
		}
	}
	//E
	end = ( x + power >= WIDTH ) ? ( WIDTH - 1 ) : ( x + power );
	for ( int i = x + 1; i <= end; ++i ){
		StaticObject& to = mStaticObjects( i, y );
		to.setFlag( StaticObject::FLAG_FIRE_X );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK | StaticObject::FLAG_BOMB ) ){ //uĂΉ΂͎~܂
			break;
		}else{
			//ACeΖE
			to.resetFlag( StaticObject::FLAG_ITEM_BOMB | StaticObject::FLAG_ITEM_POWER );
		}
	}
	//
	end = ( y - power < 0 ) ? 0 : ( y - power );
	for ( int i = y - 1; i >= end; --i ){
		StaticObject& to = mStaticObjects( x, i );
		to.setFlag( StaticObject::FLAG_FIRE_Y );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK | StaticObject::FLAG_BOMB ) ){ //uĂΉ΂͎~܂
			break;
		}else{
			//ACeΖE
			to.resetFlag( StaticObject::FLAG_ITEM_BOMB | StaticObject::FLAG_ITEM_POWER );
		}
	}
	//
	end = ( y + power >= HEIGHT ) ? ( HEIGHT - 1 ) : ( y + power );
	for ( int i = y + 1; i <= end; ++i ){
		StaticObject& to = mStaticObjects( x, i );
		to.setFlag( StaticObject::FLAG_FIRE_Y );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK | StaticObject::FLAG_BOMB ) ){ //uĂΉ΂͎~܂
			break;
		}else{
			//ACeΖE
			to.resetFlag( StaticObject::FLAG_ITEM_BOMB | StaticObject::FLAG_ITEM_POWER );
		}
	}

	//[牺͗]

	//牺
	//Ã^C~OŶ߂ɗĂ^C~OYĂ܂邽߂ɂB
	//
	//ƂāeAƂBE珇ɔƁA܂
	//
	//ƂȂBAĂB̌AĂāAĂA
	//@@
	//ƒBƁA̔Ղ̂ȂȂāA
	//
	//ƏĂĂ܂̂łBhɂ́A͘A锚e̒ōłx̂ɍ킹ďĂ˂΂ȂȂB
	//̂߂ɁAĂ͈̗̔͂͂̃JEg0ɏĂKv̂łB
	//낢Ȃ邪A΃Rsyōςނ̗̂pĂ݂B
	//łA{͔Jnɔ̋yԃ}XŒ肷@̕B
	//eNXʌpӂāAĂ}X̃Xg̂f낤B
	//͑ɂȂĂ܂̂ŁATvƂĂ͂낵ȂƎvAĂB

	//
	end = ( x - power < 0 ) ? 0 : ( x - power );
	for ( int i = x - 1; i >= end; --i ){
		StaticObject& to = mStaticObjects( i, y );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK ) ){ //e͑fʂ肷BǂAď邩炾B
			if ( ( o.mCount == 0 ) && to.checkFlag( StaticObject::FLAG_BRICK ) ){ //KȂĂJEgJn
				to.mCount = 0;
			}
			break;
		}
	}
	//E
	end = ( x + power >= WIDTH ) ? ( WIDTH - 1 ) : ( x + power );
	for ( int i = x + 1; i <= end; ++i ){
		StaticObject& to = mStaticObjects( i, y );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK ) ){ 
			if ( ( o.mCount == 0 ) && to.checkFlag( StaticObject::FLAG_BRICK ) ){ //KȂĂJEgJn
				to.mCount = 0;
			}
			break;
		}
	}
	//
	end = ( y - power < 0 ) ? 0 : ( y - power );
	for ( int i = y - 1; i >= end; --i ){
		StaticObject& to = mStaticObjects( x, i );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK ) ){ 
			if ( ( o.mCount == 0 ) && to.checkFlag( StaticObject::FLAG_BRICK ) ){ //KȂĂJEgJn
				to.mCount = 0;
			}
			break;
		}
	}
	//
	end = ( y + power >= HEIGHT ) ? ( HEIGHT - 1 ) : ( y + power );
	for ( int i = y + 1; i <= end; ++i ){
		StaticObject& to = mStaticObjects( x, i );
		if ( to.checkFlag( StaticObject::FLAG_WALL | StaticObject::FLAG_BRICK ) ){
			if ( ( o.mCount == 0 ) && to.checkFlag( StaticObject::FLAG_BRICK ) ){ //KȂĂJEgJn
				to.mCount = 0;
			}
			break;
		}
	}
}

bool State::hasCleared() const {
	//GcĂȂ΃NA
	for ( int i = 0; i < mDynamicObjectNumber; ++i ){
		if ( mDynamicObjects[ i ].isEnemy() ){
			return false;
		}
	}
	return true;
}

bool State::isAlive( int playerID ) const {
	//ΐĂ
	for ( int i = 0; i < mDynamicObjectNumber; ++i ){
		if ( mDynamicObjects[ i ].mType == DynamicObject::TYPE_PLAYER ){
			if ( mDynamicObjects[ i ].mPlayerID == playerID ){
				return true;
			}
		}
	}
	return false;
}

