#include "GameLib/Framework.h"
#include "Library/Vector3.h"
#include "Library/Vector2.h"
#include "Library/Matrix44.h"
#include "Library/Model.h"
#include "Library/Batch.h"
#include "Library/IndexBuffer.h"
#include "Library/VertexBuffer.h"
#include "Library/Camera.h"
#include "GameLib/Math.h"
using namespace GameLib;

const Vector3 gPositions[ 4 ] = {
	Vector3( -10.0, -10.0, 0.0 ),
	Vector3( -10.0, 10.0, 0.0 ),
	Vector3( 10.0, -10.0, 0.0 ),
	Vector3( 10.0, 10.0, 0.0 ),
};

const Vector2 gUVs[ 4 ] = {
	Vector2( 0.0, 0.0 ),
	Vector2( 0.0, 1.0 ),
	Vector2( 1.0, 0.0 ), 
	Vector2( 1.0, 1.0 ),
};

const unsigned gIndices[ 6 ] = {
	0, 1, 2,
	3, 1, 2,
};

bool gFirstFrame = true;
VertexBuffer* gVertexBuffer;
IndexBuffer* gIndexBuffer;
GameLib::Texture* gTexture;
Batch* gBatch;
Model* gModel;
Camera* gCamera;
int gCount;

namespace GameLib{
	void Framework::update(){
		if ( gFirstFrame ){
			setFrameRate( 60 );
			gFirstFrame = false;
			//_obt@
			gVertexBuffer = new VertexBuffer( 4 );
			for ( int i = 0; i < 4; ++i ){
				gVertexBuffer->setPosition( i, gPositions[ i ] );
				gVertexBuffer->setUV( i, gUVs[ i ] );
			}
			//CfNXobt@
			gIndexBuffer = new IndexBuffer( 6 );
			for ( int i = 0; i < 6; ++i ){
				gIndexBuffer->setIndex( i, gIndices[ i ] );
			}
			//eNX`
			createTexture( &gTexture, "grid.tga" );
			//ob`
			gBatch = new Batch( gVertexBuffer, gIndexBuffer, gTexture, BLEND_OPAQUE );
			//f
			gModel = new Model( gBatch );
			//J
			gCamera = new Camera();
		}
		//JXV
		gCamera->setPosition( Vector3(
			sin( gCount ) * 20.0,
			10.0,
			cos( gCount ) * 20.0 ) );

		Matrix44 pvm;
		gCamera->createPerspectiveViewMatrix( &pvm );
		//f̉]XV
		gModel->setAngle( Vector3( gCount, gCount * 2, gCount * 3 ) );
		//`
		gModel->draw( pvm );

		++gCount;

		//nBԂƋtɏ̂ʂ̋ZB
		//gĂɏāAgĂ̂͌ɏׂłB
		if ( isEndRequested() ){
			SAFE_DELETE( gCamera );
			SAFE_DELETE( gModel );
			SAFE_DELETE( gBatch );
			SAFE_DELETE( gIndexBuffer );
			SAFE_DELETE( gVertexBuffer );
			destroyTexture( &gTexture );
		}
	}
}
