#include "GameLib/GameLib.h"
#include "Loader.h"
#include <fstream>
using namespace std;

File::File( const char* filename ) : 
mFilename( filename ),
mData( 0 ),
mSize( 0 ){
}

File::~File(){
	if ( mData ){
		SAFE_DELETE_ARRAY( mData );
	}
	mSize = 0;
}

bool File::isReady() const {
	return ( mData != 0 ); //[hI܂Ńf[^ɒl͓Ȃ̂łŃI[P[
}

int File::getSize() const {
	ASSERT( mData && "Loading is not finished. " );
	return mSize;
}

const char* File::getData() const {
	ASSERT( mData && "Loading is not finished. " );
	return mData;
}

Loader* Loader::mInstance = 0;

Loader::Loader() : 
mCurrentFile( 0 ),
mStream( 0 ),
mFileSize( 0 ),
mFilePosition( 0 ),
mData( 0 ){
}

Loader::~Loader(){
	typedef list< File* >::iterator It;
	for ( It i = mFiles.begin(); i != mFiles.end(); ++i ){
		SAFE_DELETE( *i );
	}
	//ǂłt@CȂj
	if ( mStream ){
		SAFE_DELETE( mStream );
	}
	if ( mData ){
		SAFE_DELETE( mData );
	}
	mCurrentFile = 0; //deleteȂBXg̒ɂ͂
}

Loader* Loader::instance(){
	return mInstance;
}

void Loader::create(){
	ASSERT( !mInstance && "already created." );
	mInstance = new Loader();
}

void Loader::destroy(){
	SAFE_DELETE( mInstance );
}

void Loader::createFile( File** f, const char* filename ){
	ASSERT( !( *f ) && "pointer must be NUL. " ); 
	*f = new File( filename );
	mFiles.push_back( *f );
}

void Loader::destroyFile( File** f ){
	if ( !( *f ) ){ //ł0B邱ƂȂB
		return;
	}
	typedef list< File* >::iterator It;
	for ( It i = mFiles.begin(); i != mFiles.end(); ++i ){
		if ( *i == *f ){ //B
			SAFE_DELETE( *f ); //
			mFiles.erase( i ); //Xg
			*f = 0; //󂯎|C^0
			break;
		}
	}
	ASSERT( !( *f ) && "can't find this pointer in the list. there must be a BUG!" );
}

void Loader::update(){
	typedef list< File* >::iterator It;
	//łɃ[hȂ烍[hi߂
	if ( mCurrentFile ){
		//łA{ɃXgɂ́H
		bool found = false;
		for ( It i = mFiles.begin(); i != mFiles.end(); ++i ){
			if ( *i == mCurrentFile ){
				found = true; //
				break;
			}
		}
		if ( !found ){ //ȂȂBĂႤBrdestroy()ꂽ悤B
			if ( mData ){
				SAFE_DELETE_ARRAY( mData );
			}
			if ( mStream ){
				SAFE_DELETE( mStream );
			}
			mFileSize = mFilePosition = 0;
			mCurrentFile = 0;
		}else{ //t@CBႠ[h𑱂邩B
			int rest = mFileSize - mFilePosition;
			//ǂރTCYREAD_UNITȏȂREAD_UNITAȉȂrest̂܂
			int size = ( rest > READ_UNIT ) ? READ_UNIT : rest;
			//ǂݍ
			mStream->read( mData + mFilePosition, size );
			mFilePosition += size;
			//IȂZbg悤B
			if ( size == rest ){ //̓ǂݍݗʂƎc肪=I
				mCurrentFile->mData = mData;
				mCurrentFile->mSize = mFileSize;
				mData = 0; //ׂ̉肽̂0
				mFileSize = mFilePosition = 0;
				SAFE_DELETE( mStream );
				mCurrentFile = 0;
			}
		}
	}else{ //āAǂݍݒ̂̂ȂȂt@CJ˂΂ȂȂȂB
		for ( It i = mFiles.begin(); i != mFiles.end(); ++i ){
			File* f = *i;
			if ( !f->isReady() ){ //IĂ˂B[h
				mStream = new ifstream( f->mFilename.c_str(), ifstream::binary );
				mStream->seekg( 0, ifstream::end );
				mFileSize = static_cast< int >( mStream->tellg() );
				mData = new char[ mFileSize ];
				mStream->seekg( 0, ifstream::beg ); //ŏɊ߂ĂB
				mFilePosition = 0;
				mCurrentFile = f;
				break; //B
			}
		}
	}
}
