#include "GameLib/GameLib.h"
#include "GameLib/Threading/Functions.h"
#include "Loader.h"
#include <fstream>
using namespace std;
using namespace GameLib;
using namespace GameLib::Threading;

File::File( const char* filename ) : 
mFilename( filename ),
mData( 0 ),
mSize( 0 ){
}

File::~File(){
	if ( mData ){
		SAFE_DELETE_ARRAY( mData );
	}
	mSize = 0;
}

bool File::isReady() const {
	return ( mData != 0 ); //[hI܂Ńf[^ɒl͓Ȃ̂łŃI[P[
}

int File::getSize() const {
	ASSERT( mData && "Loading is not finished. " );
	return mSize;
}

const char* File::getData() const {
	ASSERT( mData && "Loading is not finished. " );
	return mData;
}

void LoadingThread::operator()(){
	bool end = false;
	while ( !end ){ //[v
		Threading::sleep( 1 ); //Ƃɂ΂炭QBOÔhB
		Loader::instance()->update( &end );
	}
}

Loader* Loader::mInstance = 0;

Loader::Loader() : mThread( 0 ), mEndRequest( false ){
	mThread = new LoadingThread;
	mLock = Mutex::create();

	//0
	for ( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		mFiles[ i ] = 0;
	}
}

Loader::~Loader(){
	//Xbh~߂
	mLock.lock();
	mEndRequest = true;
	mLock.unlock();
	//XbhdeleteBwait()ďI҂
	SAFE_DELETE( mThread );

	//Xg𖕎E
	for ( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if ( mFiles[ i ] ){
			SAFE_DELETE( mFiles[ i ] );
		}
	}
}

Loader* Loader::instance(){
	return mInstance;
}

void Loader::create(){
	ASSERT( !mInstance && "already created." );
	mInstance = new Loader();
}

void Loader::destroy(){
	SAFE_DELETE( mInstance );
}

void Loader::createFile( File** f, const char* filename ){
	ASSERT( !( *f ) && "pointer must be NUL. " ); 
	//󂢂ĂꏊT
	for ( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if ( !mFiles[ i ] ){
			*f = new File( filename );
			mLock.lock(); //Oɓ
			mFiles[ i ] = *f;
			mLock.unlock();
			break;
		}
	}
}

void Loader::destroyFile( File** f ){
	if ( !( *f ) ){ //ł0B邱ƂȂB
		return;
	}
	//T
	for ( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		if ( mFiles[ i ] == *f ){
			mLock.lock(); //Oɓ
			SAFE_DELETE( mFiles[ i ] );
			mLock.unlock();
			*f = 0; //󂯎|C^0
			break;
		}
	}
	ASSERT( !( *f ) && "can't find this pointer in the list. there must be a BUG!" );
}

void Loader::update( bool* endOut ){
	for ( int i = 0; i < MAX_FILE_NUMBER; ++i ){
		string filename; //t@Co߂ɁB

		mLock.lock();
		if ( mEndRequest ){ //IƌĂB̃[vŔ܂B
			*endOut = true;
		}
		if ( mFiles[ i ] && !mFiles[ i ]->isReady() ){
			filename = mFiles[ i ]->mFilename;
		}
		mLock.unlock(); //[hIǂĈo

		if ( filename.size() > 0 ){ //t@CĂB[hƂƂB
			int size;
			char* data = 0;
			//U[J̕ϐɃ[hB΃NeBJZNVɓ炸ɍςށB
			ifstream in( filename.c_str(), ifstream::binary );
			in.seekg( 0, ifstream::end );
			size = static_cast< int >( in.tellg() ); 
			in.seekg( 0, ifstream::beg );
			data = new char[ size ];
			in.read( data, size );
			//[hIĂNeBJZNVɓBςȂƌ\B
			mLock.lock();
			if ( mFiles[ i ] ){ //ȂȂB`FbNB
				mFiles[ i ]->mData = data;
				data = 0; //Ǘn̂0ɁB
				mFiles[ i ]->mSize = size;
			}
			mLock.unlock();
			if ( data ){ //f[^܂ƂƂ́AifɓȂƂƂjB
				//delete͒x̂ŃNeBJZNVɓꂽȂ̂B
				SAFE_DELETE_ARRAY( data );
			}
		}
	}
}
