#include "StdAfx.h"
#include "GameLib/GameLib.h"
#include "GameLib/Scene/CollisionDetector.h"
#include "GameLib/Base/Impl/ReferenceType.h"
#include "GameLib/Base/Tank.h"
#include "GameLib/Base/Pool.h"
#include "GameLib/Math/Vector3.h"
#include "GameLib/Math/Functions.h"

namespace {
using namespace GameLib;
using namespace GameLib::Scene;
typedef CollisionDetector::Pair Pair; //p

class Sphere{
public:
	Vector3 mCenter;
	float mRadius;
};

//Node::Build̈Zbg
class Args{
public:
	Args( 
	const Sphere* spheres,
	int sphereNumber ) : 
	mSpheres( spheres ),
	mIndexPool( sphereNumber ),
	mResults( sphereNumber ){
		mHitFlags = NEW bool[ sphereNumber * 2 ];
	}
	~Args(){
		mSpheres = 0;
		SAFE_DELETE_ARRAY( mHitFlags );
	}

	const Sphere* mSpheres;
	Pool< int > mIndexPool;
	Tank< Pair > mResults;
	bool* mHitFlags;
};

//k-d tree̖{
class Node{
public:
	Node() : mIndices( 0 ), mIndexNumber( 0 ){}
	//\zĂ͒[画肵Ă
	void build(
	const Vector3& minV, //͈͍ŏl()
	const Vector3& maxV, //͈͍ől(EO)
	Args* args,
	int restLevel ){
		//ŗǂ̕ǂ߂邩͓肾Ał͍łBꂪϓIɂ͑ÓȌʂoB
		int axis = -1; //
		Vector3 size;
		size.setSub( maxV, minV );
		float maxSize = -FLOAT_MAX;
		for ( int i = 0; i < 3; ++i ){
			if ( size[ i ] > maxSize ){
				maxSize = size[ i ];
				axis = i;
			}
		}
		//
		float div;
		div = minV[ axis ] + maxV[ axis ];
		div *= 0.5f;
		//茋ʂ͌ŎĝboolzɕێĂB̈newƒx̂ŁAłĂB
		bool* hitL = args->mHitFlags;
		bool* hitR = hitL + mIndexNumber;
		//E̐̕
		int cl = 0;
		int cr = 0;
		//ǂɑ
		const Sphere* spheres = args->mSpheres;
		for ( int i = 0; i < mIndexNumber; ++i ){ 
			const Sphere& s = spheres[ mIndices[ i ] ];
			//ɂčŏő擾
			float minP, maxP;
			minP = s.mCenter[ axis ] - s.mRadius;
			maxP = s.mCenter[ axis ] + s.mRadius;
	
			hitL[ i ] = ( minP < div ); //炩ɍɏ 덷͍lȂB̂̏ꍇ덷łǂȂĂȂAvZȂB
			hitR[ i ] = ( maxP > div ); //炩ɉEɏ
			//҂łꍇB̏ꍇEɏBɏƂԂ邩炾B
			if ( ( minP == maxP ) && ( minP == div ) ){
				hitR[ i ] = true;
			}
			if ( hitL[ i ] ){
				++cl;
			}
			if ( hitR[ i ] ){
				++cr;
			}
		}
		//Jn

		//m[h̓[JϐłBO_BƓɔsĖ߂ĂĂ܂AĂKv͂ȂB
		Node left;
		Node right;
		left.mIndices = args->mIndexPool.allocate( cl );
		right.mIndices = args->mIndexPool.allocate( cr );

		for ( int i = 0; i < mIndexNumber; ++i ){ //eOp`ɂĒׂ
			if ( hitL[ i ] ){
				left.mIndices[ left.mIndexNumber ] = mIndices[ i ];
				++( left.mIndexNumber );
			}
			if ( hitR[ i ] ){
				right.mIndices[ right.mIndexNumber ] = mIndices[ i ];
				++( right.mIndexNumber );
			}
		}
		ASSERT( ( left.mIndexNumber == cl ) && ( right.mIndexNumber == cr ) );

		//qbgtÖg܂킷֌WŁAċAĂяo͐U蕪IĂB
		if ( restLevel > 1 ){ //ŏIx͂Ă΂Ȃ
			if ( cl > 1 ){
				Vector3 boxDiv = maxV;
				boxDiv[ axis ] = div;
				left.build( minV, boxDiv, args, restLevel - 1 );
			}
			if ( cr > 1 ){
				Vector3 boxDiv = minV;
				boxDiv[ axis ] = div;
				right.build( boxDiv, maxV, args, restLevel - 1 );
			}
		}else{
			//肷B
			left.test( args );
			right.test( args );
		}
	}
	void test( Args* args ) const {
		const Sphere* spheres = args->mSpheres;

		for ( int i = 0; i < mIndexNumber; ++i ){
			int i0 = mIndices[ i ];
			const Sphere& s0 = spheres[ mIndices[ i ] ];
			const Vector3& p0 = s0.mCenter;
			float r0 = s0.mRadius;
			for ( int j = i + 1; j < mIndexNumber; ++j ){
				int i1 = mIndices[ j ];
				const Sphere& s1 = spheres[ mIndices[ j ] ];
				const Vector3& p1 = s1.mCenter;
				float r1 = s1.mRadius;
				//B덷͍lƂǂ
				Vector3 d;
				d.setSub( p1, p0 );
				float rsum = r0 + r1;
				float rsum2 = rsum * rsum;
				float dist2 = d.squareLength();
				if ( dist2 <= rsum2 ){
					Pair* pair = args->mResults.add();
					if ( i0 < i1 ){ //mId0ƂB
						pair->mId0 = i0;
						pair->mId1 = i1;
					}else{
						pair->mId0 = i1;
						pair->mId1 = i0;
					}
				}
			}
		}
	}
	int* mIndices;
	int mIndexNumber;
};

} //namespace {}

namespace GameLib{
namespace Scene{

class CollisionDetector::Impl : public ReferenceType{
public:
	Impl( int capacity, int maxLevel ) : 
	mSpheres( 0 ),
	mSphereNumber( 0 ),
	mCapacity( capacity ),
	mMaxLevel( maxLevel ){
		mSpheres = NEW Sphere[ mCapacity ];
	}
	~Impl(){
		SAFE_DELETE_ARRAY( mSpheres );
	}
	int add( const Vector3& c, float r ){
		int i = mSphereNumber;
		ASSERT( i < mCapacity );
		mSpheres[ i ].mCenter = c;
		mSpheres[ i ].mRadius = r;
		++mSphereNumber;
		return i;
	}
	void detect( Array< Pair >* resultsOut ){
#if 1
		Node root;
		Args args( mSpheres, mSphereNumber );
		root.mIndexNumber = mSphereNumber;
		root.mIndices = args.mIndexPool.allocate( mSphereNumber );
		//m[hɃf[^[UBɔ͈͂vZ
		Vector3 minV( FLOAT_MAX ); //őŏ
		Vector3 maxV( -minV.x ); //ŏŏ
		for ( int i = 0; i < mSphereNumber; ++i ){
			const Sphere& s = mSpheres[ i ];
			Vector3 tMin = s.mCenter;
			Vector3 tMax = s.mCenter;
			Vector3 r( s.mRadius );
			tMin -= r;
			tMin += r;
			minV.min( tMin ); //minXYZōŏlĎɓB
			maxV.max( tMax ); //maxXYZōőlĎɓB
			//ԍ[U
			root.mIndices[ i ] = i;
		}
		//m[h͋Âɓ鐔pӂ邪Ał炭ς邾낤B
		//nȏŏ2̙p߁A̓{pӂ΂悢B
		int n = mSphereNumber;
		int maxNodeNumber = 1;
		int maxLevel = 0; //ő啪
		while ( maxNodeNumber < n ){
			maxNodeNumber <<= 1;
			++maxLevel;
		}
		root.build( minV, maxV, &args, maxLevel );
		//Tank̏dB
		//1.ANZX邽߂ɔzɈڂ
		int resultSize = args.mResults.size();
		Pair* resultsArray = NEW Pair[ resultSize ];
		int pos = 0;
		while ( !args.mResults.isEnd() ){
			resultsArray[ pos ] = *args.mResults.get();
			++pos;
			args.mResults.toNext();
		}
		//2.Ԗڂił̂̂𐔂B
		int* counts = NEW int[ mSphereNumber * 2 ];
		int* offsets = counts + mSphereNumber; //ꏏɊmہB₱ƂƃoǑɂȂ
		for ( int i = 0; i < mSphereNumber; ++i ){ //[v
			counts[ i ] = 0;
		}
		for ( int i = 0; i < resultSize; ++i ){
			++counts[ resultsArray[ i ].mId0 ];
		}
		//2.ItZbgɕϊ
		int offset = 0;
		for ( int i = 0; i < mSphereNumber; ++i ){
			offsets[ i ] = offset;
			offset += counts[ i ];
			counts[ i ] = 0;
		}
		//1.\[gzɃ\[gȂi[
		int* sorted = NEW int[ offset ]; //offsetɂ傤ǍvĂB
		for ( int i = 0; i < resultSize; ++i ){
			int i0 = resultsArray[ i ].mId0;
			sorted[ offsets[ i0 ] + counts[ i0 ] ] = resultsArray[ i ].mId1;
			++counts[ i0 ];
		}
		//Ƃɏd̂߂lɃZbg
		resultsOut->setSize( offset ); //d݂œ鐔m
		pos = 0;
		for ( int i = 0; i < mSphereNumber; ++i ){
			int* box = &sorted[ offsets[ i ] ];
			int n = counts[ i ];
			for ( int j = 0; j < n; ++j ){
				if ( box[ j ] >= 0 ){
					( *resultsOut )[ pos ].mId0 = i;
					( *resultsOut )[ pos ].mId1 = box[ j ];
					++pos;
					for ( int k = j + 1; k < n; ++k ){ //̂ɂ΂ԂB
						if ( box[ j ] == box[ k ] ){
							box[ k ] = -1;
						}
					}
				}
			}
		}
		//ŏIʂ̃TCYkBz͋NȂB
		resultsOut->setSize( pos );
		//n
		SAFE_DELETE_ARRAY( sorted );
		SAFE_DELETE_ARRAY( counts );
		SAFE_DELETE_ARRAY( resultsArray );
#else //
		Tank< Pair > results( mSphereNumber );
		for ( int i = 0; i < mSphereNumber; ++i ){
			const Sphere& s0 = mSpheres[ i ];
			const Vector3& p0 = s0.mCenter;
			float r0 = s0.mRadius;
			for ( int j = i + 1; j < mSphereNumber; ++j ){
				const Sphere& s1 = mSpheres[ j ];
				const Vector3& p1 = s1.mCenter;
				float r1 = s1.mRadius;
				//B덷͌ōl
				Vector3 d;
				d.setSub( p1, p0 );
				float rsum = r0 + r1;
				float rsum2 = rsum * rsum;
				float dist2 = d.getSquareLength();
				if ( dist2 <= rsum2 ){
					Pair* pair = results.add();
					pair->mId0 = i;
					pair->mId1 = j;
				}
			}
		}
		//zɕϊ
		results.copyTo( resultsOut );
#endif
		//ɋ̌0ɁB
		mSphereNumber = 0;
	}
private:
	void operator=( const Impl& ); //֎~

	Sphere* mSpheres;
	int mSphereNumber;
	const int mCapacity;
	const int mMaxLevel;
};

CollisionDetector::CollisionDetector( Impl* impl ) : mImpl( impl ){
	if ( mImpl ){
		mImpl->refer();
	}
}

CollisionDetector CollisionDetector::create( int capacity, int maxLevel ){
	CollisionDetector r;
	r.mImpl = NEW Impl( capacity, maxLevel );
	return r;
}

int CollisionDetector::add( const Vector3& sphereCenter, float sphereRadius ){
	return mImpl->add( sphereCenter, sphereRadius );
}

void CollisionDetector::detect( Array< Pair >* resultOut ){
	mImpl->detect( resultOut );
}

#define TYPE CollisionDetector
#include "GameLib/Base/Impl/ReferenceTypeTemplate.h"

} //namespace Scene
} //namespace GameLib
