#ifndef INCLUDED_GAMELIB_VECTORIMPL_H
#define INCLUDED_GAMELIB_VECTORIMPL_H

#include "GameLib/GameLib.h"
#include "GameLib/Base/Vector.h"

namespace GameLib{

template< class T > inline Vector< T >::Vector( int size, int capacity ) : 
mElements( 0 ),
mSize( size ),
mCapacity( capacity ){
	if ( capacity <= 0 ){
		mCapacity = mSize;
	}
	if ( mSize > 0 ){
		mElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		for ( int i = 0; i < mSize; ++i ){
			new ( &mElements[ i ] ) T;
		}
	}
}

template< class T > inline Vector< T >::Vector( const Vector& a ) : 
mElements( 0 ),
mSize( a.mSize ),
mCapacity( a.mCapacity ){
	if ( mSize > 0 ){
		mElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		for ( int i = 0; i < mSize; ++i ){
			new ( &mElements[ i ] ) T( a.mElements[ i ] ); //Rs[RXgN^Ăяo
		}
	}
}

template< class T > inline Vector< T >::~Vector(){
	clear();
}

template< class T > inline void Vector< T >::operator=( const Vector& a ){ 
	//operator=()ɂ͎P[XɋC悤B
	if ( this == &a ){
		return;
	}
	//܂ۂƍ폜B\L̈悪ĂAʂɂȂ\̂ō폜B
	clear();

	mSize = a.size;
	mCapacity = a.mCapacity;
	if ( mSize > 0 ){
		mElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		for ( int i = 0; i < mSize; ++i ){
			new ( &mElements[ i ] ) T( a.mElements[ i ] ); //Rs[RXgN^Ăяo
		}
	}
}


template< class T > inline void Vector< T >::push( const T& a ){
	if ( mSize == mCapacity ){ //ꏊȂBzsB
		//1.5{+1TCŸ̗mہB+12ł悤ɂ邽߁B
		mCapacity = ( mCapacity * 3 / 2 ) + 1;
		T* newElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		//SRs[ьÂvf̔j
		for ( int i = 0; i < mSize; ++i ){
			new ( &newElements[ i ] ) T( mElements[ i ] );
			mElements[ i ].~T();
		}
		//̈̔j
		OPERATOR_DELETE( mElements );
		//ւ
		mElements = newElements;
	}
	//Vvf̒ǉ
	new ( &mElements[ mSize ] ) T( a );
	//TCYXV
	++mSize;
}

template< class T > inline T* Vector< T >::push(){
	if ( mSize == mCapacity ){ //ꏊȂBzsB
		//1.5{+1TCŸ̗mہB+12ł悤ɂ邽߁B
		mCapacity = ( mCapacity * 3 / 2 ) + 1;
		T* newElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		//SRs[ьÂvf̔j
		for ( int i = 0; i < mSize; ++i ){
			new ( &newElements[ i ] ) T( mElements[ i ] );
			mElements[ i ].~T();
		}
		//̈̔j
		OPERATOR_DELETE( mElements );
		//ւ
		mElements = newElements;
		newElements = 0;
	}
	//Vvf̒ǉ
	new ( &mElements[ mSize ] ) T;
	//TCYXV
	++mSize;
	//Vvf̃|C^Ԃ
	return &mElements[ mSize - 1 ];
}

template< class T > inline void Vector< T >::clear(){
	//fXgN^Ăяo
	for ( int i = 0; i < mSize; ++i ){
		mElements[ i ].~T();
	}
	//̈J
	OPERATOR_DELETE( mElements );
	mSize = mCapacity = 0;
}

template< class T > inline int Vector< T >::size() const {
	return mSize;
}

template< class T > inline void Vector< T >::setSize( int n ){
	//wTCYcapacityȉȂzȂ
	if ( n < mCapacity ){
		for ( int i = n; i < mSize; ++i ){ //͕̉̕KvB53ɂȂY3,4͉Kv
			mElements[ i ].~T();
		}
	}else if ( n > mCapacity ){ //_Bz
		mCapacity = n;
		T* newElements = static_cast< T* >( OPERATOR_NEW( sizeof( T ) * mCapacity ) );
		//SRs[ьÂvf̔j
		for ( int i = 0; i < mSize; ++i ){
			new ( &newElements[ i ] ) T( mElements[ i ] );
			mElements[ i ].~T();
		}
		//̈̔j
		OPERATOR_DELETE( mElements );
		//ւ
		mElements = newElements;
		newElements = 0;
		//Ȃ̈ɃftHgRXgN^Ăяo
		for ( int i = mSize; i < n; ++i ){
			new ( &mElements[ i ] ) T;
		}
	}
	mSize = n;
}

template< class T > inline T& Vector< T >::operator[]( int i ){
	ASSERT( i >= 0 && i < mSize && "Range Error!" );
	return mElements[ i ];
}

template< class T > inline const T& Vector< T >::operator[]( int i ) const {
	ASSERT( i >= 0 && i < mSize && "Range Error!" );
	return mElements[ i ];
}

template< class T > inline void Vector< T >::moveTo( Vector* a ){
	a->clear();
	a->mElelents = mElements;
	a->mSize = mSize;
	a->mCapacity = mCapacity;
	mElements = 0;
	mSize = mCapacity = 0;
}

} //namespace GameLib

#endif
