#ifndef INCLUDED_GAMELIB_GRAPHICS_MANAGER_H
#define INCLUDED_GAMELIB_GRAPHICS_MANAGER_H

#include "GameLib/Graphics/Enum.h"

namespace GameLib{

namespace Math{
	class Matrix44;
	class Matrix34;
	class Vector2;
	class Vector3;
}
using namespace Math;
	
namespace Graphics{

class Texture;
class VertexBuffer;
class IndexBuffer;

class Manager{
public:
	Manager();
	static Manager instance();

	void setVertexBuffer( VertexBuffer );
	void setIndexBuffer( IndexBuffer );
	void setTexture( Texture );

	void enableDepthTest( bool );
	void enableDepthWrite( bool );
	void setBlendMode( BlendMode );
	void setCullMode( CullMode );
	void setTextureFilter( TextureFilter );
	///ϊr[ϊsZbgB
	void setProjectionViewMatrix( const Matrix44& );
	///[hϊsZbg
	void setWorldMatrix( const Matrix34& );
	///r[|[gݒ
	void setViewport( int x, int y, int width, int height );
	///r[|[g擾(0łȂ̂ɂ̂ŗ~̂|C^Zbg)
	void getViewport( int* xOut, int* yOut, int* widthOut, int* heightOut );

	///ݒ肵eNX`A_obt@ACfNXobt@ŕ`
	void drawIndexed(
		int offset, 
		int primitiveNumber, 
		PrimitiveType = PRIMITIVE_TRIANGLE );
	///ݒ肵eNX`A_obt@ŕ`
	void draw( 
		int offset, 
		int primitiveNumber, 
		PrimitiveType = PRIMITIVE_TRIANGLE );
	///SʂɃeNX`\t
	void blendToScreen( Texture );
	///tXN[
	void enableFullScreen( bool );
	///`obt@Ԃ
	int width() const;
	///`obt@Ԃ
	int height() const;
	///tXN[̃EBhEԂ
	int fullScreenWindowWidth() const;
	///tXN[̃EBhEԂ
	int fullScreenWindowHeight() const;
	///t[ԍԂ
	unsigned frameId() const;
	///XN[Vbg(gqDDS,TGA)Bۑ^C~O͕`IB
	void captureScreen( const char* filename );

	//CeBO֌W
	///sNZVF[_L
	void setLightingMode( LightingMode );
	///Cgʒu
	void setLightPosition( int index, const Vector3& position );
	///Cgx(100Ȃ100m̋1̖邳ɂȂBxIƂāA邳́uI/vŌ܂)
	void setLightIntensity( int index, float intensity );
	///CgF
	void setLightColor( int index, const Vector3& );
	///_ʒu
	void setEyePosition( const Vector3& position );
	///fBt[YJ[
	void setDiffuseColor( const Vector3& );
	///XyLF
	void setSpecularColor( const Vector3& );
	///F
	void setEmissionColor( const Vector3& );
	///ArGgF
	void setAmbientColor( const Vector3& );
	///xݒ
	void setTransparency( float );
	///XyL̉s(傫قǉsB̈Ӗ͕ʂ̖{ǂ)
	void setSpecularSharpness( float );

	//ȉmȂĂ
	static void create( 
		void* windowHandle,
		int width,
		int height,
		bool fullScreen,
		bool vSync,
		bool antiAlias );
	static void destroy();
	void beginDraw();
	void endDraw();
	bool canRender() const; //foCXXgĂfalseԂB
	void restore(); //foCXXg畜A
	void getPointerModifier( float* scale, Vector2* offset ) const;
};

} //namespace Graphics
} //namespace GameLib

#endif
