#include "GameLib/GameLib.h"
#include "XFile/Template.h"
#include "XFile/Token.h"
#include "XFile/Element.h"
#include "GameLib/Base/Tank.h"
#include "XFile/Parser.h"
#include <sstream>

namespace XFile{

using namespace GameLib;
using namespace std;

Template::Template( RefString name, const RefString& uuid, int childNumber ) :
mName( name ),
mChildren( childNumber ), 
mUuid( uuid ){
}

Template::~Template(){
}

void Template::setChild( int i, TemplateLine** tl ){
	mChildren[ i ] = *tl;
	*tl = 0;
}

const RefString* Template::getType() const {
	return &mName;
}

const RefString* Template::getChildName( int index ) const {
	if ( index >= mChildren.size() ){ //傫ԍv邱ƂBgB0ԂB
		return 0;
	}else{
		return &mChildren[ index ]->mName;
	}
}

const RefString* Template::getChildType( int index ) const {
	if ( index >= mChildren.size() ){ //傫ԍv邱ƂBgB0ԂB
		return 0;
	}else if ( mChildren[ index ]->mTemplate ){ //^
		return mChildren[ index ]->mTemplate->getType();
	}else{ //v~eBu
		return &mChildren[ index ]->mPrimitiveType;
	}
}


int Template::createElement( 
Element** elementOut, 
const Array< Token >& t,
int p,
const Parser& parser,
const RefString& name,
map< RefString, Element* >* namedElements,
int* level ) const {

//if ( p % 1000 == 0 ){ cout << p << endl; }

	const int n = t.size();
	Tank< CompositeElement::Child > children;
	map< RefString, int > arrayIndices;
	//M[f[^
	for ( int i = 0; i < mChildren.size(); ++i ){
		++( *level );
		CompositeElement::Child c; //q
		if ( ( p < n ) && ( t[ p ] == Token::LEFT_BRACE ) ){ //QƗ܂...
			p = parseReference( &c.mElement, t, p, namedElements );
		}else{
			const TemplateLine& line = *mChildren[ i ];
			if ( line.mIndices.size() > 0 ){ //zł
				p = createArray( &c.mElement, line, t, p, arrayIndices, parser, namedElements, level );
			}else if ( line.mTemplate ){ //^ł
				RefString nullStr;
				p = line.mTemplate->createElement( &c.mElement, t, p, parser, nullStr, namedElements, level );
			}else{ //v~eBuł
				c.mPrimitive = *t[ p ].get();
				++p;
				int size = parser.toInt( c.mPrimitive ); //YɂȂ肤邩ׂ܂
				if ( size > 0 ){
					arrayIndices.insert( make_pair( line.mName, size ) );
				}
			}
		}
		children.add( c );
		Parser::dieIf( p >= n );
		if ( t[ p ] == Token::SEMICOLON ){
			++p;
		}else{ //肦Ȃ
#if 0 //^ZRCAΉBZ~RX[
			Parder::dieIf( true );
#endif
		}
		--( *level );
#if 1 //^ZRCAΉR[h ]ȃZ~R
		if ( ( *level == 0 ) && ( p < n ) && ( t[ p ] == Token::SEMICOLON ) ){ //肦Ȃ
			++p;
		}
#endif
	}
	//ǉf[^
	while ( p < n ){
		CompositeElement::Child c;
		if ( t[ p ] == Token::LEFT_BRACE ){ //QƗ܂...
			p = parseReference( &c.mElement, t, p, namedElements );
		}else if ( t[ p ] == Token::IDENTIFIER ){
			//^o܂
			const RefString* typeName = t[ p ].get();
			++p;
			//ϐo
			RefString childName;
			if ( ( p < n ) && ( t[ p ] == Token::IDENTIFIER ) ){
				childName = *t[ p ].get();
				++p;
			}
			//{
			Parser::dieIf( ( p >= n ) || ( t[ p ] != Token::LEFT_BRACE ) ); //肦Ȃ
			++p;
			//ev[gɃf[^Ă炢܂B
			if ( parser.isPrimitiveType( *typeName ) ){ //v~eBu
				c.mPrimitive = *t[ p ].get();
				++p;
			}else{ //^
				const Template* tmpl = parser.getTemplate( *typeName );
				int childLevel = 0;
				p = tmpl->createElement( &c.mElement, t, p, parser, childName, namedElements, &childLevel );
				namedElements->insert( make_pair( childName, c.mElement ) );
			}
			//}
			Parser::dieIf( ( p >= n ) || ( t[ p ] != Token::RIGHT_BRACE ) ); //肦Ȃ
			++p;
		}else{
			break; //QƂłf[^̂łȂB܂IB
		}
		children.add( c );
	}
	CompositeElement* e = NEW CompositeElement( this, children.size(), name );
	for ( int i = 0; !children.isEnd(); ++i ){
		CompositeElement::Child* c = children.get();
		e->setChild( i, *c );
		children.toNext();
	}
	*elementOut = e;

	return p;
}

int Template::createArray(
Element** elementOut, 
const TemplateLine& line,
const Array< Token >& t,
int p,
const map< RefString, int >& arrayIndices,
const Parser& parser,
map< RefString, Element* >* namedElements,
int* level ) const {
	//ŏɂ邱Ƃ͔z̓Y̎ƊẽTCY擾邱ƁB͍œK̗]n
	const int n = t.size();
	const Array< RefString >& indices = line.mIndices;
	int dimension = indices.size();
	Array< int > arraySizes( dimension );
	int k = 0;
	int arraySize = 1;
	for ( int i = 0; i < indices.size(); ++i ){
		const RefString& s = indices[ i ];
		int size = parser.toInt( s );
		if ( size > 0 ){
			arraySizes[ k ] = size;
		}else{
			map< RefString, int >::const_iterator l = arrayIndices.find( s );
			Parser::dieIf( l == arrayIndices.end() );
			arraySizes[ k ] = l->second;
		}
		arraySize *= arraySizes[ k ];
	}
	//v~eBuz񂩕^ŕ
	if ( line.mTemplate ){ //^
		CompositeArrayElement* a = NEW CompositeArrayElement( arraySizes );
		for ( int i = 0; i < arraySize; ++i ){
			Element* te = 0;
			RefString nullStr;
			p = line.mTemplate->createElement( &te, t, p, parser, nullStr, namedElements, level );
			if ( i < arraySize - 1 ){ //ŏIvf̓J}ȂBȂĂI
				Parser::dieIf( ( p >= n ) || ( t[ p ] != Token::COMMA ) );
				++p;
			}
			a->setElement( i, &te );
		}
		*elementOut = a;
	}else{ //v~eBu
		PrimitiveArrayElement* a = NEW PrimitiveArrayElement( arraySizes, line.mPrimitiveType );
		for ( int i = 0; i < arraySize; ++i ){
			a->setElement( i, *t[ p ].get() );
			++p;
			if ( i < arraySize - 1 ){ //ŏIvf̓J}ȂBȂĂI
				Parser::dieIf( ( p >= n ) || ( t[ p ] != Token::COMMA ) );
				++p;
			}
		}
		*elementOut = a;
	}
	return p;
}

//QƏ { name | UUID | name UUID }
int Template::parseReference(
Element** elementOut,
const Array< Token >& t,
int p,
map< RefString, Element* >* ) const {
	const int n = t.size();

	++p; //΂
	RefString refName;
	if ( ( p < n ) && ( t[ p ] == Token::IDENTIFIER ) ){
		refName = *t[ p ].get();
		++p;
	}
	if ( ( p < n ) && ( t[ p ] == Token::UUID ) ){ //UUID͖OȂĂB
		++p;
		if ( refName.size() == 0 ){
			refName = *t[ p ].get();
		}
	}
	Parser::dieIf( refName.size() == 0 );
	Parser::dieIf( ( p >= n ) || ( t[ p ] != Token::RIGHT_BRACE ) ); //肦Ȃ
	++p;

	*elementOut = NEW ReferenceElement( refName );
/*
	//QƂ݂܂
	map< RefString, Element* >::iterator it = namedElements->find( refName );
	Parser::dieIf( it == namedElements->end() );
	*elementOut = it->second;
*/
	return p;
}


} //namespace XFile

